@@ -1,18 +1,10 @@
 #include "config.h"
 #include "crypto/bn.h"
+#include "tl-parser/portable_endian.h"
 #include "tgl.h"
 #include "tools.h"
 #include "mtproto-utils.h"
 
-#if !defined(HAVE___BUILTIN_BSWAP32) && !defined(__FreeBSD__) && !defined(__OpenBSD__)
-static inline unsigned __builtin_bswap32(unsigned x) {
-	  return ((x << 24) & 0xff000000 ) |
-		    ((x << 8) & 0x00ff0000 ) |
-		      ((x >> 8) & 0x0000ff00 ) |
-		        ((x >> 24) & 0x000000ff );
-}
-#endif
-
 static unsigned long long gcd (unsigned long long a, unsigned long long b) {
   return b ? gcd (b, a % b) : a;
 }
@@ -105,22 +97,25 @@ int tglmp_check_g_a (struct tgl_state *TLS, TGLC_bn *p, TGLC_bn *g_a) {
 static unsigned long long BN2ull (TGLC_bn *b) {
   if (sizeof (unsigned long) == 8) {
     return TGLC_bn_get_word (b);
+  } else if (sizeof (unsigned long long) == 8) {
+    unsigned long long tmp;
+    /* Here be dragons, but it should be okay due to be64toh */
+    TGLC_bn_bn2bin (b, (unsigned char *) &tmp);
+    return be64toh (tmp);
   } else {
-    unsigned int tmp[2];
-    memset (tmp, 0, 8);
-    TGLC_bn_bn2bin (b, (void *)tmp);
-    return __builtin_bswap32 (tmp[0]) * (1ll << 32) | __builtin_bswap32 (tmp[1]);
+    assert (0);
   }
 }
 
 static void ull2BN (TGLC_bn *b, unsigned long long val) {
   if (sizeof (unsigned long) == 8 || val < (1ll << 32)) {
     TGLC_bn_set_word (b, val);
+  } else if (sizeof (unsigned long long) == 8) {
+    val = htobe64(val);
+    /* Here be dragons, but it should be okay due to htobe64 */
+    TGLC_bn_bin2bn ((unsigned char *) &val, 8, b);
   } else {
-    unsigned int tmp[2];
-    tmp[0] = __builtin_bswap32 (val >> 32);
-    tmp[1] = __builtin_bswap32 ((unsigned)val);
-    TGLC_bn_bin2bn ((void *)tmp, 8, b);
+    assert (0);
   }
 }
 
